package com.yuandian.dataflow.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Queue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.locks.ReentrantLock;

import com.alibaba.nacos.shaded.io.grpc.netty.shaded.io.netty.util.internal.shaded.org.jctools.queues.MessagePassingQueue.Consumer;
import com.google.protobuf.Any;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class PacketsManager {

    private ReentrantLock lock = new ReentrantLock();

    private LinkedBlockingQueue<Any> temp = new LinkedBlockingQueue<>();
    private LinkedBlockingQueue<Any> packets = new LinkedBlockingQueue<>();


        /**
     * 添加数据到队列里, 做了双队列缓冲
     * @param _packets
     */
    public void addPacket(Any packet) {
        try {
            lock.lock();
            this.temp.add(packet);
        } finally{
            lock.unlock();  
        }
    }


    /**
     * 添加数据到队列里, 做了双队列缓冲
     * @param _packets
     */
    public void addPackets(Collection <? extends Any> _packets) {
        
        try {
            lock.lock();
            this.temp.addAll(_packets);
        } finally{
            lock.unlock();  
            
        }
        
        
    }

    /**
     * 返回 总共packets的数据
     * @return
     */
    public int size() {
        return this.temp.size() + this.packets.size();
    }

    private void lockTemp( Consumer< LinkedBlockingQueue<Any>> dofunc) {
        try {
            lock.lock();
            this.packets.addAll(this.temp); // 合并缓冲的数据到使用中. 
            this.temp.clear();
            dofunc.accept(this.packets);
        } catch (Exception e) {
            log.error("{}",e);
        } finally {
            lock.unlock();
           
        }
    }

    /**
     * 弹出需要数量num的数据
     * @param num 需要数量 
     * @return 反回弹出少于等于num的数据
     */
    public ArrayList<Any>  popPackets(int num) {
        var result = new ArrayList<Any>();
        this.lockTemp((_packets)->{
            for(var i = 0 ; i < num ; i++) {
                var p = _packets.poll();
                if(p == null){
                    return;
                }
                result.add(p);
            }
        });
        return result;
    }

    /**
     * 丢弃数据, 保持数据量在一定值
     * @param remainSize 保留范围
     */
    public void discardPackets(int remainSize) {
        this.lockTemp((_packets)->{
            var delsize =  _packets.size() - remainSize ;
            var step = _packets.size() / delsize ;
            var iter = _packets.iterator();
            for(var i = 0 ; i < packets.size(); i++) {
                iter.next();
                if(i % step == 0) {
                    iter.remove();
                }
            }
        });
    }
}
